package com.smartgwt.sample.server;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import com.isomorphic.datasource.DSRequest;
import com.isomorphic.datasource.DSResponse;
import com.isomorphic.sql.SQLDataSource;
import com.isomorphic.util.ISCDate;

public class LongDataSource extends SQLDataSource
{
	private final static int MILLISECONDS_IN_DAY = 1000*60*60*24;
	
	private static final long serialVersionUID = 1L;
	
	private static Logger logger = Logger.getLogger(LongDataSource.class.getName ());

	public DSResponse execute(DSRequest request) throws Exception 
	{
		if (request.getOperationType().equals("update"))
		{
			// get the entire list of fields in this DataSource
			List fields = request.getDataSource().getFieldNames();
			
			// create a DSRequest to fetch the record we have to update and execute it
			DSRequest first = new DSRequest();
			first.setDataSource(request.getDataSource());
			first.setCriteria(request.getCriteria());
			first.setOperationType("fetch");
			DSResponse record = first.execute();
					
			// check if there was any change in database while the user was editing the data
			// for each key we have to check if is defined in both nw and old and also if their 
			// value is same. 
			Map old = request.getOldValues();
			Map nw = record.getRecord();
			
			Iterator it = fields.iterator();
			while( it.hasNext())
			{
				String key = (String)it.next();
				
				if (!compareRecordValues(old.get(key), nw.get(key)))
				{	
					logger.info("Difference key="+key+" old="+old.get(key)+" new="+nw.get(key));
					
					// Difference was found. 
					// return the latest record, so the client side can display it to the user
					record.setStatus(-74);
					return record;
				}
			}
		}

		// if no change was found or the operation is not update, execute the standard code
		return super.execute(request);
	} 
	
	/**
	 * Compare two record values to check if they're same or not. Comparing them is not easy since both 
	 * values can be different type representing the same thing, for example Integer and String with same value
	 * or Boolean and String, or ISCDate and Date, or Date and String and so on, so conversions are required
	 * in order to be able to compare them
	 * 
	 * @param v1 first value to be checked
	 * @param v2 second value to be checked
	 * 
	 * @return true if the two values match, false otherwise
	 */
	private boolean compareRecordValues(Object v1, Object v2)
	{
		// if both are null, they're equal
		if (v1 == null && v2 == null)
		{
			return true;
		}
		
		String c1 = "null";
		String c2 = "nul";
		if (v1 != null)
		{
			c1 = v1.getClass().getName();
		}
		if (v2 != null)
		{
			c2 = v2.getClass().getName();
		}
		
		logger.info("Comparing "+v1+" type of "+c1+" with "+v2+" type of "+c2);
		
		// if one of them is null and the other isn't, then they're not equal
		if ((v1 != null && v2==null ) || (v1 == null && v2 != null))
		{
			return false;
		}
		
		// if both of them have the same class, compare them with equals()
		if (v1.getClass() == v2.getClass())
		{
			logger.info("Both values have same class, comparing with equals()");
			
			return v1.equals(v2);
		}
		
		// if one of items is ISCDate, convert both of them to ISCDate 
		if (v1 instanceof ISCDate|| v2 instanceof ISCDate)
		{
			ISCDate d1 = null;
			ISCDate d2 = null;
			
			logger.info("Converting both of them to ISCDate before compare");
			// v1 is ISCDate, convert v2
			if (v1 instanceof ISCDate)
			{
				d1 = (ISCDate)v1;
				
				if ( v2 instanceof Date)
				{
					d2 = new ISCDate(((Date)v2).getTime());
				}
				else
				{
					// not date, might be string or something else?
					d2 = new ISCDate(new Date(v2.toString()).getTime());
				}
			}
			else
			{
				// v2 is ISCDate, convert v1
				d2 = (ISCDate)v2;
				
				if ( v1 instanceof Date)
				{
					d1 = new ISCDate(((Date)v1).getTime());
				}
				else
				{
					// not date, might be string or something else?
					d1 = new ISCDate(new Date(v1.toString()).getTime());
				}
			}
			
			logger.info("d1="+d1.toString()+" d2="+d2.toString());
			
			if (Math.abs(d1.getTime()-d2.getTime()) < MILLISECONDS_IN_DAY)
			{
				return true;
			}
			
			return false;
		}
		
		// if one of them is Long, convert both to Long
		if (v1 instanceof Long || v2 instanceof Long)
		{
			Long d1 = null;
			Long d2 = null;
			
			logger.info("Converting both of them to Long before compare");
			// v1 is Long, convert v2
			if (v1 instanceof Long)
			{
				d1 = (Long)v1;
				d2 = Long.parseLong(v2.toString());
			}
			else
			{
				// v2 is Long, convert v1
				d1 = (Long)v2;				
				d1 = Long.parseLong(v1.toString());
			}
			
			if (d1.equals(d2))
			{
				return true;
			}
			
			return false;
		}

		// if one of them is Integer, convert both to Integer
		if (v1 instanceof Integer || v2 instanceof Integer)
		{
			Integer d1 = null;
			Integer d2 = null;
			
			logger.info("Converting both of them to Integer before compare");
			// v1 is Integer, convert v2
			if (v1 instanceof Long)
			{
				d1 = (Integer)v1;
				d2 = Integer.parseInt(v2.toString());
			}
			else
			{
				// v2 is Integer, convert v1
				d2 = (Integer)v2;				
				d1 = Integer.parseInt(v1.toString());
			}
			
			if (d1.equals(d2))
			{
				return true;
			}
			
			return false;
		}

		// if one of them is Double, convert both to Double
		if (v1 instanceof Double || v2 instanceof Double)
		{
			Double d1 = null;
			Double d2 = null;
			
			logger.info("Converting both of them to Double before compare");
			// v1 is Double, convert v2
			if (v1 instanceof Double)
			{
				d1 = (Double)v1;
				d2 = Double.parseDouble(v2.toString());
			}
			else
			{
				// v2 is Double, convert v1
				d2 = (Double)v2;				
				d1 = Double.parseDouble(v1.toString());
			}
			
			if (d1.equals(d2))
			{
				return true;
			}
			
			return false;
		}
		
		// if one of them is Boolean, convert both to Boolean
		if (v1 instanceof Boolean || v2 instanceof Boolean)
		{
			Boolean d1 = null;
			Boolean d2 = null;
			
			logger.info("Converting both of them to Boolean before compare");
			// v1 is Boolean, convert v2
			if (v1 instanceof Boolean)
			{
				d1 = (Boolean)v1;
				d2 = Boolean.parseBoolean(v2.toString());
			}
			else
			{
				// v2 is Boolean, convert v1
				d2 = (Boolean)v2;				
				d1 = Boolean.parseBoolean(v1.toString());
			}
			
			if (d1.equals(d2))
			{
				return true;
			}
			
			return false;
		}
		
		// if no one of previous conditions were true, probably the fields don't match
		return false;
	}
}